<?php

    if ( !defined( 'ABSPATH' ) ) exit;

    class JDownloadFromFiles617 extends stdClass
    {
        private $options = null;
        private $template_folder = '';
        private $date_format = '';
        private $time_format = '';
        private $disabled = array();
        
        public function __construct($options)
        {
            $this->options = $options;
            if ((!isset($this->options['token'])) or (empty($this->options['token']))) {
                $this->options['token'] = $this->getToken();
            }
            
            if (!isset($this->options['show_folders'])) {
                $this->options['show_folders'] = $this->options['folders'];
            }

            if (!class_exists ('JDovi42Files', false)) {
                include_once(dirname(__file__) .DS. 'jdovi42_files.php');
            }
            
            $this->date_format = get_option('date_format');
            $this->time_format = get_option('time_format');

            if (isset($this->options['disabled'])) {
                $this->disabled = $this->options['disabled'];
                unset($this->options['disabled']);
            }
            

        }
        
        function getFiles($target) {
            
            $target = str_replace('/', DIRECTORY_SEPARATOR, $target);

            $source = new JDovi42Files( $target );
            $files = array();
            $source->allowExt = explode(',', $this->options['accept']);
            $source->addTilt($this->disabled);
            
            if ($this->options['sort_files'] != 'set') {
                $source->filesSort = 'SortBy' . ucfirst($this->options['sort_files']);
            }
            if ($this->options['sort_order'] == 'asc') { $source->filesSortOrder = 'asc'; }
            if ($this->options['sort_order'] == 'desc') { $source->filesSortOrder = 'desc'; }

            $files = $source->getFiles();

            if (!empty($files)) {

                $uploaders = null;
                if (file_exists($target .DIRECTORY_SEPARATOR. 'upload.log')) {
                    $uploaders = new JDovi42Files( $target, 'upload.log');
                }

                for ($i = 0; $i <= count($files)-1; $i++) {
                	if ((isset($files[$i]['ext'])) and (!isset($this->options['icon']))) {
                        $files[$i]['icon'] = $this->getIcon($files[$i]['ext']);
                    }                
                    if ($this->options['hide_direct_url'] == 1) {
                        $files[$i]['script'] = 'onclick="download_from_files_617_download_file(\''.base64_encode(str_replace(DOWNLOAD_FROM_FILES_617_WP_ROOT, '', $target) .'/'.$files[$i]['filename']).'\');return false;"';
                        $files[$i]['url'] = '#';
                    } else {
                        $files[$i]['script'] = '';
                        $files[$i]['url'] = esc_url(home_url('/')) . str_replace(DOWNLOAD_FROM_FILES_617_WP_ROOT, '', $target) .'/'.$files[$i]['filename'];
                    }
                    $files[$i]['path'] = str_replace(DOWNLOAD_FROM_FILES_617_WP_ROOT, '', $target) .'/'.$files[$i]['filename'];
                    $files[$i]['download_title'] = sprintf( __('%s download', 'download-from-files'), $files[$i]['title'] );
                    
                    $files[$i]['download_text'] = __('Download', 'download-from-files');

                	if (!empty($uploaders)) {
                        $uploader = explode('|', $uploaders->getTitle($files[$i]['filename']));
                        if ($uploader[0] != $files[$i]['filename']) {
                            $files[$i]['uploader'] = $uploader[0]; 
                        }
                    }
                }
            }
            return $files;
        }
        
        function getFolders($target) {
            
            if ((isset($this->options['show_folders'])) and ($this->options['show_folders'] != 1)) {
                return array();
            }
            $source = new JDovi42Files( $target );
            $source->addTilt($this->disabled);

            if ($this->options['sort_folders'] != 'set') {
                $source->folderSort = 'SortBy' . ucfirst($this->options['sort_folders']);
            }

            if ($this->options['sort_order'] == 'asc') { $source->folderSortOrder = 'asc'; }
            if ($this->options['sort_order'] == 'desc') { $source->folderSortOrder = 'desc'; }

            $folders = $source->getFolders();
            if (!empty($folders)) {
                for ($i = 0; $i <= count($folders)-1; $i++) {
                    $folders[$i]['icon'] = 'folder.png';
                    $folders[$i]['folders'] = $this->getFolders($target .DIRECTORY_SEPARATOR. $folders[$i]['filename'] );
                    $folders[$i]['files'] = $this->getFiles($target .DIRECTORY_SEPARATOR. $folders[$i]['filename']);
                    $folders[$i] = $this->prepareData($folders[$i]);
                }
            }

            return $folders;
        }
        

        private function getToken()
        {
            $chars = 'ABCDEFGHIJKLMNOPQURSTXYVWZ0123456789';
            $token = '';
            for ($i = 1; $i <= 10; $i++) {
            	$token .= substr($chars, rand(0, strlen($chars)-1), 1);
            }                
            return $token;
        }


        function showError($text, $info) {
            $html = '';
            if (!empty($text)) {
                $html = '<div class="download-from-files-warning">' . "\r\n" .
                        sprintf( $text, $info ). "\r\n" .
                     '</div>' . "\r\n";
            }
            return $html;
        }

        function prepareData($data) {
            
            if ((empty($data['folders'])) and (empty($data['files']))) {
                $data['empty'] = $this->options['empty_text'];
                $data['count'] = 0;
            } else {
                $data['empty'] = 0;
                if (!empty($data['files'])) {
                    $data['count'] = count($data['files']);
                } else {
                    $data['count'] = 0;
                }
            }
            if (!empty($data['folders'])) {
                for ($i = 0; $i <= count($data['folders'])-1; $i++) {
                    if (has_filter('download_from_files_counter')) {
                        $data['folders'][$i]['count'] = apply_filters('download_from_files_counter', $data['folders'][$i]['count']);
                    } else {
                        $data['folders'][$i]['count'] = '';
                    }

                    if (has_filter('download_from_files_icon')) {
                        if (isset($data['folders'][$i]['icon'])) {
                            $data['folders'][$i]['icon'] = apply_filters('download_from_files_icon', $data['folders'][$i]['icon']);
                        } 
                    } else {
                        $data['folders'][$i]['icon'] = '';    
                    }
                }
            }            

            if (!empty($data['files'])) {
                for ($i = 0; $i <= count($data['files'])-1; $i++) {
                    
                    if (has_filter('download_from_files_date')) {
                        $data['files'][$i]['date'] = apply_filters('download_from_files_date', $data['files'][$i]['date'], $this->date_format, $this->time_format);
                    } else {
                        $data['files'][$i]['date'] = '';    
                    }

                    if (has_filter('download_from_files_uploader')) {
                        if (isset($data['files'][$i]['uploader'])) {
                            $data['files'][$i]['uploader'] = apply_filters('download_from_files_uploader', $data['files'][$i]['uploader']);
                        }
                    } else {
                        $data['files'][$i]['uploader'] = '';    
                    }
    
                    if (has_filter('download_from_files_icon')) {
                        if (isset($data['files'][$i]['icon'])) {
                            $data['files'][$i]['icon'] = apply_filters('download_from_files_icon', $data['files'][$i]['icon']);
                        } 
                    } else {
                        $data['files'][$i]['icon'] = '';    
                    }

                }
            }
            
            return $data;
        }

        function getOneFile($file) {

            $files = $this->getFiles(dirname(DOWNLOAD_FROM_FILES_617_WP_ROOT . $this->options['target']));
            $file = $files[ array_search( pathinfo(DOWNLOAD_FROM_FILES_617_WP_ROOT . $this->options['target'], PATHINFO_BASENAME), array_column($files, 'filename') ) ];
            
            $tmp = array();
            $tmp['files'][] = $file;
            $tmp = $this->prepareData($tmp);
            
            return $tmp['files'][0];
        }

        function prepareContent() {   

            $html = '';

            if (!class_exists('JDovi42_Templates')) {
                include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'jdovi42_templates.php');
            }

            if (is_file(DOWNLOAD_FROM_FILES_617_WP_ROOT . $this->options['target'])) {

                if ((!isset($this->options['template'])) or (empty($this->options['template']))) {
                    $this->options['template'] = 'default';
                }
                
                $data = $this->getOneFile($this->options['target']);
                $data['template'] = $this->options['template'];

                // $tmpl = new JDovi42_Templates($this->options['template'].'/file.html');
                $tmpl = new JDovi42_Templates(DOWNLOAD_FROM_FILES_617_ROOT);
                $tmpl->useTemplate($this->options['template'].'/file.html');
                $tmpl->setOptions($data);
                $tmpl->setOptions($this->options);
                $html = $tmpl->prepareContent();
                
            } else {

                if ((!isset($this->options['template'])) or (empty($this->options['template']))) {
                    $this->options['template'] = 'default';
                }
                // $tmpl = new JDovi42_Templates($this->options['template']);
                $tmpl = new JDovi42_Templates(DOWNLOAD_FROM_FILES_617_ROOT);
                $tmpl->useTemplate($this->options['template'].'/index.html');
                
                if ($tmpl->checkTemplate()) {
                    $this->options['options'] = base64_encode(json_encode($this->options));
    
                    $this->options['folders'] = $this->getFolders(DOWNLOAD_FROM_FILES_617_WP_ROOT . $this->options['target']);
                    $this->options['files'] = $this->getFiles(DOWNLOAD_FROM_FILES_617_WP_ROOT . $this->options['target']);
                    $this->options = $this->prepareData($this->options);

                    $tmpl->setOptions($this->options);
                    $html = $tmpl->prepareContent();
                } else {
                    $html = '<div class="download-from-files-warning">' . "\r\n" .
                            sprintf( __('Template file not exists! <small>(%s)</small>', 'download-from-files'), $this->options['template'] ). "\r\n" .
                         '</div>' . "\r\n";
                }
            }
            
            return $html;
        }


        function getIcon($ext)
        {
            $icon = '';
            
            switch ($ext){ 
            	case 'exe':
            	   $icon = 'exe.png';
                   break;
        
            	case 'doc':
            	case 'docx':
            	case 'odt':
            	   $icon = 'doc.png';
                   break;
        
            	case 'xls':
            	case 'xlsx':
            	case 'ods':
            	case 'xml':
            	case 'csv':
            	   $icon = 'xls.png';
                   break;
        
            	case 'ppt':
            	case 'pptx':
            	   $icon = 'ppt.png';
                   break;
        
            	case 'sql':
            	case 'dump':
            	   $icon = 'sql.png';
             	   break;
        
            	case 'zip':
            	case 'rar':
            	case 'arj':
            	case 'tgz':
            	   $icon = 'package.png';
                   break;
            
            	case 'txt':
            	   $icon = 'text.png';
                   break;
            
            	case 'pdf':
            	   $icon = 'pdf.png';
                   break;
        
            	case 'gif':
            	case 'jpg':
            	case 'png':
            	   $icon = 'img.png';
                   break;
        
        
            	default :
                    $icon = 'file.png';
            }            
            
            return $icon;
        }


         
    }