<?php

if ( !defined( 'ABSPATH' ) ) exit;

class download_from_files_617_settings
{
    private $options;

    public function __construct()
    {
        add_action( 'admin_menu', array( $this, 'add_plugin_page' ) );
        add_action( 'admin_init', array( $this, 'page_init' ) );
    }

    public function add_plugin_page()
    {
        // This page will be under "Settings"
        add_options_page(
            'Settings Admin', 
            __('Download from files settings', 'download-from-files'), 
            'manage_options', 
            'download-from-files-setting-admin', 
            array( $this, 'create_admin_page' )
        );
    }

    public function create_admin_page()
    {
        $this->options = download_from_files_617_get_options ( 'download_from_files_options' );
        ?>
        <div class="wrap">
            <h1><?php echo __('Download from files settings', 'download-from-files');?></h1>
            <?php
                $active_tab = isset( $_GET[ 'download-from-files-tab' ] ) ? sanitize_text_field($_GET[ 'download-from-files-tab' ]) : 'files';
            ?>
            <h2 class="nav-tab-wrapper">
                <a href="?page=download-from-files-setting-admin&download-from-files-tab=files" class="nav-tab  <?php echo $active_tab == 'files' ? 'nav-tab-active' : ''; ?>"><?php echo __('Files', 'download-from-files');?></a>
                <a href="?page=download-from-files-setting-admin&download-from-files-tab=settings" class="nav-tab  <?php echo $active_tab == 'settings' ? 'nav-tab-active' : ''; ?>"><?php echo __('Plugin settings', 'download-from-files');?></a>
                <a href="?page=download-from-files-setting-admin&download-from-files-tab=readme" class="nav-tab  <?php echo $active_tab == 'readme' ? 'nav-tab-active' : ''; ?>"><?php echo __('Using plugin', 'download-from-files');?></a>
            </h2>
            <?php
                switch ($active_tab){ 

                    case 'settings':
                        ?>
                        <form method="post" action="options.php">
                        <?php
                            // This prints out all hidden setting fields
                            settings_fields( 'download_from_files_option_main' );
                            do_settings_sections( 'download-from-files-setting-admin' );
                            submit_button();
                        ?>
                        </form>
                        <?php
                        exit;

                    case 'readme':
                        echo '<h3>' . __('So use it!', 'download-from-files'). '</h3>';
                        include( dirname(__FILE__) . DIRECTORY_SEPARATOR . 'how_to_use.php' );
                        /*
                            or 
                            include( dirname(__FILE__) . DIRECTORY_SEPARATOR . 'how_to_use_'.get_bloginfo('language').'.php' );
                            // include ./how_to_use_en.php
                        */
                        exit;
                        
                    default: 
                        include( dirname(__FILE__) . DS . 'downloads.php' );
                        $options = download_from_files_617_get_options();
                        $downloads = new JDownloadsFromFiles($options);
                        
                        $downloads->render();
                    
                }
            ?>
        </div>
        <?php
    }

    /**
     * Register and add settings
     */
    public function page_init()
    {        
        register_setting(
            'download_from_files_option_main', // Option group
            'download_from_files_options', // Option name
            array( $this, 'sanitize' ) // Sanitize
        );

        add_settings_section(
            'download_from_files_settings_main', // ID
            __('Download from files Custom Settings', 'download-from-files'), // Title
            array( $this, 'print_section_info' ), // Callback
            'download-from-files-setting-admin' // Page
        );  

        add_settings_field(
            'template', // ID
            __('Set default template', 'download-from-files'), // Title
            array( $this, 'template_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );

        add_settings_field(
            'folders', // ID
            __('Show folders', 'download-from-files'), // Title
            array( $this, 'folders_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );

        add_settings_field(
            'empty_text', // ID
            __('Empty text', 'download-from-files'), // Title
            array( $this, 'empty_text_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );


        add_settings_field(
            'accept', // ID
            __('Accept types', 'download-from-files'), // Title
            array( $this, 'accept_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );

        add_settings_field(
            'disabled', // ID
            __('Disabled(s)', 'download-from-files'), // Title
            array( $this, 'disabled_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );

        add_settings_field(
            'sort_folders', // ID
            __('Sort folders', 'download-from-files'), // Title
            array( $this, 'sort_folders_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );

        add_settings_field(
            'sort_files', // ID
            __('Sort files', 'download-from-files'), // Title
            array( $this, 'sort_files_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );

        add_settings_field(
            'sort_order', // ID
            __('Sort order', 'download-from-files'), // Title
            array( $this, 'sort_order_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );

        add_settings_field(
            'hide_direct_url', // ID
            __('Hide direct url', 'download-from-files'), // Title
            array( $this, 'hide_direct_url_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );

        add_settings_field(
            'rename', // ID
            __('Rename file', 'download-from-files'), // Title
            array( $this, 'rename_callback' ), // Callback
            'download-from-files-setting-admin', // Page
            'download_from_files_settings_main' // Section
        );

    }

    public function sanitize( $input )
    {
        $new_input = array();

        if ( isset( $input['template'] ) ) {
            $new_input['template'] = trim( $input['template'] );
        }
        if ( isset( $input['folders'] ) ) {
            $new_input['folders'] = 1;
        } else {
            $new_input['folders'] = 0;
        }
        if ( isset( $input['empty_text'] ) ) {
            $new_input['empty_text'] = trim( $input['empty_text'] );
        }
        if ( isset( $input['accept'] ) ) {
            $new_input['accept'] = trim( str_replace(' ', '', $input['accept'] ));
        }
        if ( isset( $input['disabled'] ) ) {
            $new_input['disabled'] = trim( str_replace(' ', '', $input['disabled'] ));
        }
        if ( isset( $input['sort_folders'] ) ) {
            $new_input['sort_folders'] = trim( $input['sort_folders'] );
        }
        if ( isset( $input['sort_files'] ) ) {
            $new_input['sort_files'] = trim( $input['sort_files'] );
        }
        if ( isset( $input['sort_order'] ) ) {
            $new_input['sort_order'] = trim( $input['sort_order'] );
        }
        if ( isset( $input['hide_direct_url'] ) ) {
            $new_input['hide_direct_url'] = 1;
        } else {
            $new_input['hide_direct_url'] = 0;
        }

        if ( isset( $input['rename'] ) ) {
            $new_input['rename'] = 1;
        } else {
            $new_input['rename'] = 0;
        }

        return $new_input;
    }

    public function print_section_info()
    {
        print __('These are the default settings that you can overwrite when invited.', 'download-from-files');
    }

    public function template_callback()
    {

        if (!class_exists('JDovi42Files')) {
            include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'jdovi42_files.php');
        }

        $source = new JDovi42Files(get_stylesheet_directory() . '/download-from-files');
        $tmp = $source->getFolders();        
        if (!empty($tmp)) {
            for ($i = 0; $i <= count($tmp)-1; $i++) {
            	$folders[$tmp[$i]['filename']] = 'Theme: ' . __($tmp[$i]['title'], 'download-from-files'); 
            }
        }

        $source = new JDovi42Files(DOWNLOAD_FROM_FILES_617_ROOT . '/templates'); 
        $tmp = $source->getFolders();        
        if (!empty($tmp)) {
            for ($i = 0; $i <= count($tmp)-1; $i++) {
            	if (!isset($folders[$tmp[$i]['filename']])) {
                    $folders[$tmp[$i]['filename']] = __($tmp[$i]['title'], 'download-from-files'); 
            	}
            }
        }
        ksort($folders);

        $html = '<select name="download_from_files_options[template]">';
        foreach ($folders as $key => $value) {
            $html .= '<option value="'.$key.'"';
            if ($this->options['template'] == $key) {
               $html .= ' SELECTED';
            }
            $html .= '>'.$value.'</option>';
        }
        $html .= '</select>';
        echo $html;
    }


    public function folders_callback()
    {
        if ($this->options['folders'] == 1) {
            $checked = ' CHECKED ';
        } else {
            $checked = '';
        } 
        printf(
            '<input type="checkbox" name="download_from_files_options[folders]" value="1" '.$checked.' />' .
            '<br /><small>'.__('Show folders in actual folder.', 'download-from-files').'</small>',
            isset( $this->options['folders'] ) ? esc_attr( $this->options['folders']) : ''
        );
    }

    public function empty_text_callback()
    {
        printf(
            '<input type="text" name="download_from_files_options[empty_text]" value="%s" />' .
            '<br /><small>'.__('Appears only if there are no folders or files in the folder.<br />If left blank, it will not appear at all.', 'download-from-files').'</small>',
            isset( $this->options['empty_text'] ) ? esc_attr( $this->options['empty_text']) : ''
        );
    }


    public function accept_callback() {
        printf(
            '<input type="text" name="download_from_files_options[accept]" value="%s" />' .
            '<br /><small>'.__('Comma separated accept extensions.', 'download-from-files').'</small>',
            isset( $this->options['accept'] ) ? esc_attr( $this->options['accept']) : ''
        );
    }

    public function disabled_callback() {
        
        $values = $this->options['disabled'];
        $values = array_diff($values, array('wp-admin', 'wp-includes'));
        $values = implode(',', $values);
        printf(
            '<input type="text" name="download_from_files_options[disabled]" value="%s" />' .
            '<br /><small>'.__('Comma separated disabled name(s).', 'download-from-files').'</small>',
            $values
        );
    }


    public function sort_folders_callback()
    {
        echo '<!-- ' . __('set_title', 'download-from-files') . ' -->';
        echo '<!-- ' . __('name_title', 'download-from-files') . ' -->';
        echo '<!-- ' . __('title_title', 'download-from-files') . ' -->';
        echo '<!-- ' . __('date_title', 'download-from-files') . ' -->';
        $values = array('set', 'name', 'title', 'date');
        $html = '<select name="download_from_files_options[sort_folders]">';
        foreach ($values as $item) {
            $html .= '<option value="'.$item.'"';
            if ($this->options['sort_folders'] == $item) {
               $html .= ' SELECTED';
            }
            $html .= '>'.__($item.'_title', 'download-from-files').'</option>';
        }
        $html .= '</select>';
        $html .= '<br /><small>'.__('Sort folders mode', 'download-from-files').'</small>';
        echo $html;
    }

    public function sort_files_callback()
    {
        $values = array('set', 'name', 'title', 'date', 'size', 'ext');
        echo '<!-- ' . __('size_title', 'download-from-files') . ' -->';
        echo '<!-- ' . __('ext_title', 'download-from-files') . ' -->';
        $html = '<select name="download_from_files_options[sort_files]">';
        foreach ($values as $item) {
            $html .= '<option value="'.$item.'"';
            if ($this->options['sort_files'] == $item) {
               $html .= ' SELECTED';
            }
            $html .= '>'.__($item.'_title', 'download-from-files').'</option>';
        }
        $html .= '</select>';
        $html .= '<br /><small>'.__('Sort files mode', 'download-from-files').'</small>';
        echo $html;
    }

    public function sort_order_callback()
    {
        $values = array('default', 'asc', 'desc');
        echo '<!-- ' . __('default_title', 'download-from-files') . ' -->';
        echo '<!-- ' . __('asc_title', 'download-from-files') . ' -->';
        echo '<!-- ' . __('desc_title', 'download-from-files') . ' -->';
        $html = '<select name="download_from_files_options[sort_order]">';
        foreach ($values as $item) {
            $html .= '<option value="'.$item.'"';
            if ($this->options['sort_order'] == $item) {
               $html .= ' SELECTED';
            }
            $html .= '>'.__($item.'_title', 'download-from-files').'</option>';
        }
        $html .= '</select>';
        $html .= '<br /><small>'.__('Sort ordering', 'download-from-files').'</small>';
        echo $html;
    }


    public function hide_direct_url_callback()
    {
        if ($this->options['hide_direct_url'] == 1) {
            $checked = ' CHECKED ';
        } else {
            $checked = '';
        } 
        printf(
            '<input type="checkbox" name="download_from_files_options[hide_direct_url]" value="1" '.$checked.' />' .
            '<br /><small>'.__('Hide direct url in download item.', 'download-from-files').'</small>',
            isset( $this->options['hide_direct_url'] ) ? esc_attr( $this->options['hide_direct_url']) : ''
        );
    }

    public function rename_callback()
    {
        if ($this->options['rename'] == 1) {
            $checked = ' CHECKED ';
        } else {
            $checked = '';
        } 
        printf(
            '<input type="checkbox" name="download_from_files_options[rename]" value="1" '.$checked.' />' .
            '<br /><small>'.__('If you turn it on, changing the description will also change the file name.', 'download-from-files').'</small>',
            isset( $this->options['rename'] ) ? esc_attr( $this->options['rename']) : ''
        );
    }

}
