<?php

    if ( !defined( 'ABSPATH' ) ) exit;

    if (!class_exists('JDovi42_Templates')) {

        /*
         * Dear WordPress Plugin Review Team!
         * 
         * In accordance with the principles cited, the name is structured as follows:
         * "J" + "<my_registered_name_on_the_WP>" + "_" + "<a_feature_content>"
         */

        /**
         * JDovi42_Templates
         * 
         * HTML sablon használata adattartalom megjelenítéséhez WordPress rendszerben
         * A sablonok a plugin mappájában egy "templates" mappában, sablononként külön mappában,
         * vagy a WordPress (child) Téma mappájában a plugin nevével azonos mappában, azon belül pedig külön mappákban kell lenniük.
         * 
         * include_once(dirname(__file__) .DIRECTORY_SEPARATOR. 'jdovi42_templates.php');
         * 
         * $tmpl = new JDovi42_Templates(PLUGIN_ROOT_PATH);
         * $tmpl->useTemplate('default/index.html'); 
         * if ($tmpl->checkTemplate()) {
         *    $tmpl->setOptions($options);
         *    $tmpl->setGlobals($globals);
         *    $html = $tmpl->prepareContent();
         * } else {
         *    // Template not found 
         *    $html = $tmpl->getErrors();
         *    // $html = $tmpl->getErrorsHTML();
         * }
         *
         * @package template  
         * @author Dovi42
         * @copyright Minden jog fenntartva 
         * @version 2.03
         * @access public
         */
        class JDovi42_Templates extends stdClass {
            // A plugin azonosítója
            private $ID = '';
            // A használt sablon HTML fájl
            private $template_file = 'index.html';
            // A használt sablon mappája
            private $template_folder = 'default';
            // Paraméterek, melyeket a sablonban be kell helyettesíteni
            private $options = array();
            // Paraméterek, melyeket a sablonban, de mindenütt be kell helyettesíteni
            private $globals = array();
            // Hibaüzenetek tömbje
            private $errors = array();
            // Generált tartalom
            private $content = '';
            // A plugin elérési útja a fájlrendszerben
            private $plugin_path = '';
            // Az egyes sablonok elérési útja (CSS és JS beillesztéshez)
            private $template_sources  = array();
            // ABSPATH for Debian users
            private $abs_path = null;
            // used CSS list
            private $css = array();
            
            /**
             * JDovi42Templates::__construct()
             * 
             * @param text $ID Egyedi azonosító, amelyet a HTML blokk megjelenítéséhez használ 
             * @param array $options A feldolgozandó adatok tömbben 
             * @return HTML a generált HTML tartalom
            */
            public function __construct($plugin = null) {
                
                // Plugin elérési útja meghatározása
                if (empty($plugin)) {
                    $this->plugin_path = plugin_dir_path(dirname(__file__));
                } else {
                    $this->plugin_path = $plugin;
                }
                
                // Azonosító meghatározása a plugin elérési útjából.
                $this->ID = explode(DIRECTORY_SEPARATOR, rtrim($this->plugin_path, DIRECTORY_SEPARATOR));
                $this->ID = array_pop($this->ID);
                
                $abs = explode('/', WP_PLUGIN_DIR);
                unset($abs[count($abs)-1]);unset($abs[count($abs)-1]);
                $this->abs_path = implode('/', $abs) .DIRECTORY_SEPARATOR;

                $this->template_file = 'index.html';
                $this->template_folder = '';

                // Alapértelmezett értékek hozzáadása
                $this->setDefaults();
    
            }

            /**
             * JDovi42_Templates::useTemplate()
             * Set the template file
             * 
             * @param string $file file path and name
             * @return 
             */
            function useTemplate($file = '') {
                if (!empty($file)) {

                    $file = explode('/', $file);
                    if (count($file) > 1) {
                        $this->template_file = $file[count($file)-1];  
                        unset($file[count($file)-1]);  
                        $this->template_folder = implode(DIRECTORY_SEPARATOR, $file) .DIRECTORY_SEPARATOR;
                    } else {
                        $this->template_file = $file[0];
                    }
                }
            }
    
            /**
             * JDovi42Templates::checkTemplate()
             * 
             * @return (bool) Template available status
             */
            function checkTemplate() {
                $found = false;
                if (file_exists(get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $this->template_folder . $this->template_file)) {
                    $found = true;
                } else {
                    if (file_exists($this->plugin_path .DIRECTORY_SEPARATOR. 'templates' .DIRECTORY_SEPARATOR. $this->template_folder . $this->template_file)) {
                        $found = true;
                    }
                }
                
                if (!$found) {
                    $this->addError( __('Template not available', '.$this->ID.') . ' ('.$this->template_folder.'/'.$this->template_file.')' );
                }
                
                return $found;
            }
    
            /**
             * JDovi42Templates::setOptions()
             * 
             * @param array $options
             * @return void
             */
            function setOptions($options) {
                $this->options = wp_parse_args($options, $this->options);
                // Adatok előfeldolgozása
                $this->options = $this->preProcess($this->options);
                
            }
    
            /**
             * JDovi42Templates::setGlobals()
             * 
             * @param array $options
             * @return void
             */
            function setGlobals($options) {
                $this->globals = wp_parse_args($this->globals, $options);
                
                // Adatok előfeldolgozása
                $this->globals = $this->preProcess($this->globals);
                
            }
    
            /**
             * JDovi42Templates::preProcess()
             * 
             * Az adatok előfeldolgozását végzi - módosítja az $options tömböt.
             * Akkor lehet fontos, ha pl. az egyik változó megléte esetén generálni kell egy másik változót is
             * 
             * @param array $options A módosítandó adat-tömb
             * @return void
             */
            function preProcess($options) {
                return $options;
            }
    
            /**
             * JDovi42Templates::setDefaults()
             * 
             * Alapértelmezett változó értékek hozzáadása és ezáltal a sablonban használhatóság biztosítása
             * Ha nincs rá szükség, érdemes felülírni, hogy gyorsítsa a végrehajtást.
             * Ugyancsak felülírandó, ha saját alapértelmezett értékeket akarunk hozzáadni.
             * 
             * @return void
             */
            function setDefaults() {
                
                global $wp, $post;
                
                $date_format = get_option( 'date_format' );
                $time_format = get_option( 'time_format' );
                
                // A honlap neve
                if (!isset($this->options['blog_name'])) {
                    $this->options['blog_name'] = get_bloginfo('name');
                }
                // A honlap címoldala linkje
                if (!isset($this->options['home_url'])) {
                    $this->options['home_url'] = esc_url( home_url( '/' ) );
                }
                // Az aktuális oldal linkje
                if (!isset($this->options['current_url'])) {
                    $this->options['current_url'] = esc_url( home_url( $wp->request . '/') );
                }
                // Az aktuális oldal címe
                if (!isset($this->options['current_title'])) {
                    $this->options['current_title'] = get_the_title();
                }
                // Pillanatnyi dátum
                if (!isset($this->options['date'])) {
                    $this->options['date'] = date_i18n( $date_format );
                }
                // Pillanatnyi idő
                if (!isset($this->options['time'])) {
                    $this->options['time'] = date_i18n( $time_format );
                }
                // Pillanatnyi dátum és idő
                if (!isset($this->options['datetime'])) {
                    $this->options['datetime'] = date_i18n( $date_format .' ' . $time_format);
                }
                // Plugin elérési útja
                if (!isset($this->options['plugin_root'])) {
                    // $this->options['plugin_root'] = str_replace(DOWNLOAD_FROM_FILES_617_WP_ROOT, '', $this->plugin_path);
                    $this->options['plugin_root'] = str_replace($this->abs_path, '', $this->plugin_path);
                }
            }
    
    
            /**
             * JDovi42Templates::getTemplate()
             * 
             * Megkeresi, hogy hol van a fájlrendszerben a használandó sablon és visszaadja azt
             * 
             * @param text $filename A betöltendő fájl neve
             * @param boolean $load_default Ha nem található, akkor betöltse-e a "default" sablonból
             * @return HTML sablon tartalma
             */
            function getTemplate($filename = 'index.html') {
                
                $template_source = '';
                
                if (file_exists(get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $this->template_folder . $filename)) {
                    // Elsőként a (child) témában keresi
                    $template = file_get_contents(get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $this->template_folder .$filename);
                    $info = pathinfo(get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $this->template_folder .$filename);
                    $template_source = get_stylesheet_directory() .DIRECTORY_SEPARATOR. $this->ID .DIRECTORY_SEPARATOR. $this->template_folder;
                } else {
                    if (file_exists($this->plugin_path .DIRECTORY_SEPARATOR. 'templates' .DIRECTORY_SEPARATOR. $this->template_folder . $filename)) {
                        // Majd a plugin mappájában keresi
                        $template = file_get_contents($this->plugin_path .DIRECTORY_SEPARATOR. 'templates' .DIRECTORY_SEPARATOR. $this->template_folder . $filename);
                        $info = pathinfo($this->plugin_path .DIRECTORY_SEPARATOR. 'templates' .DIRECTORY_SEPARATOR. $this->template_folder . $filename);
                        $template_source = $this->plugin_path .DIRECTORY_SEPARATOR. 'templates' .DIRECTORY_SEPARATOR. $this->template_folder;
                    } else {
                        $this->addError( __('Template not available', '.$this->ID.') . ' ('.$this->template_folder.'/'.$filename.')');
                        return '';
                    }
                }
    
                // Ha üres a sablon, akkor egy hibaüzenetet állítunk be.
                if (empty($template)) {
                    $this->addError( __('Template is empty', '.$this->ID.') . ' ('.$this->template_folder.'/'.$filename.')');                
                }
                
                // CSS és JS fájlok betöltéséhez a vizsgálandó mappák kijelölése
                // Ha az aktuális sablon mappája nincs a listában, felvesszük.
                if (!empty($template_source) and (!in_array($template_source, $this->template_sources))) {
                    $this->template_sources[] = $template_source;   
                }
    
                // Get includes in template
                $template = $this->includeTemplate($template);
    
                return $template;
                
            }
    
            /**
             * JDovi42Templates::includeTemplate()
             * 
             * Include more HTML files
             * 
             * @param mixed $template HTML content
             * @return HTML
             */
            function includeTemplate($template) {
    
                $includes = array();
                // Get the includes files list
                $regex = '/<!-- include (\s*.*?) -->/s';
                preg_match_all( $regex, $template, $matches);
    
                if (!empty($matches)) {
                    foreach ($matches[1] as $file) {
                        if (!in_array($file, $includes)) {
                            $includes[] = $file;
                        }
                    }
                }
                
                foreach ($includes as $file) {
                    $tmpl = $this->getTemplate($file);    
                    $tmpl = $this->renderContent($tmpl);   
                    $template = str_replace('<!-- include '.$file.' -->', $tmpl, $template);         
                }
                
                return $template;
            }
    
    
            function getCSS() {
                return $this->css;
            }
    
            /**
             * JDovi42Templates::loadCSSandJS()
             * 
             * CSS és JS fájlok betöltése a mappákból
             * 
             * @return void
             */
            function loadCSSandJS() {
                
                $css = array();
                $js  = array();
    
                // Minden kijelölt mappát végignézünk
                foreach ($this->template_sources as $folder) {
    
                    // Relatív útvonal kiszámítása ehhez a mappához
                    $url = esc_url( home_url( '/' ) );
                    $url = $url . str_replace($this->abs_path, '', $folder);
    
                    $handle = @opendir($folder);
                    if ($handle) {
                        while (false !== ($file = readdir($handle))) {
                            // Csak akkor vizsgálódunk, ha ez egy fájl
                            if (!is_dir($folder . $file)) {
                                $info = pathinfo($file);
                                // Csak a CSS és JS fájlokat dolgozzuk fel.
                                if (in_array($info['extension'], array('css', 'js'))) {
                                    // Fájl hozzáadása a css fájlokhoz
                                    if ($info['extension'] == 'css') {
                                        $css[] = $url . $file;
                                        $this->css[] = array(
                                                'file' => $folder .DIRECTORY_SEPARATOR. $file,
                                                'url'  => $url .DIRECTORY_SEPARATOR. $file
                                            );
                                    }
                                    // Fájl hozzáadása a css fájlokhoz
                                    if ($info['extension'] == 'js') {
                                        $js[] = $url . $file;
                                    }
                                }
                            }
                        }
                    }
                    
                }
                
                // Ha vannak CSS fájlok, akkor azok mindegyikét betöltjük
                if (!empty($css)) {
                    
                    for ($i = 0; $i <= count($css)-1; $i++) {
                        if (!wp_style_is($this->ID.'-style-'.$this->template_folder.'-'.$i)) {
                            wp_register_style( $this->ID.'-style-'.$this->template_folder.'-'.$i, $css[$i] );
                        	wp_enqueue_style(  $this->ID.'-style-'.$this->template_folder.'-'.$i );
                        }
                    }
                }
                
                // Ha vannak JS fájlok, akkor azok mindegyikét betöltjük
                if (!empty($js)) {
                    for ($i = 0; $i <= count($js)-1; $i++) {
                        if (!wp_script_is($this->ID.'-js-'.$this->template_folder.'-'.$i)) {
                            wp_register_script($this->ID.'-js-'.$this->template_folder.'-'.$i, $js[$i], array('jquery'));  
                            wp_enqueue_script($this->ID.'-js-'.$this->template_folder.'-'.$i);
                        }
                    }
                }
                
            }
    
            /**
             * JDovi42Templates::renderArray()
             * 
             * Egy tömböt renderel le
             * 
             * @param mixed $key A sablon nevét adó kulcs
             * @param mixed $data A behelyettesítendő adatok
             * @return void
             */
            private function renderArray($key, $data) {
    
                $template = $this->getTemplate($key.'.html');
                $arr = array();
                
                foreach ($data as $item) {
    
                    $item = wp_parse_args($item, $this->globals);
    
                    $temp = $template;
                    $temp = $this->conditionalTemplate($temp, $item);
                    
                    $item = $this->preProcess($item);
                    
                    foreach ($item as $k => $v) {
    
                        // Ha ez egy objektum, akkor tömböt csinálunk belőle.
                        if (is_object($v)) { $v = (array) $v; }
                        
                        if (is_array($v)) {
                            // Ha ez egy tömb
                            $html = $this->renderArray($k, $v);
                            $temp = str_replace('<!-- ' . $k . ' -->', $html, $temp);
                        } else {
                            $temp = str_replace('<!-- ' .$k. ' -->', $this->modifyValue($k, $v), $temp);
                        }
                    }
                    $arr[] = $temp;
                }
                return implode('', $arr);
            }
    
            /**
             * JDovi42Templates::renderContent()
             * 
             * HTML tartalom renderelése, a változók behelyettesítése
             * 
             * @param mixed $template
             * @return void
             */
            private function renderContent($template) {
                
                // Ha üres a sablon, akkor üres a renderelt tartalom is
                if (empty($template)) { return ''; }
                
                // Feltételes formázásnak megfelelő sablon módosítás
                $template = $this->conditionalTemplate($template);
    
                $options = wp_parse_args($this->options, $this->globals);
    
                foreach ($options as $key => $value) {
                // Adatok behelyettesítése
    
                    // Ha ez egy objektum, akkor tömböt csinálunk belőle.
                    if (is_object($value)) { $value = (array) $value; }
    
                    if (is_array($value)) {
                        // Ha ez egy tömb, akkor azt más eljárással generáljuk
                        $html = $this->renderArray($key, $value);
                        $template = str_replace('<!-- ' . $key . ' -->', $html, $template);
                    } else {
                        $template = str_replace('<!-- ' . $key . ' -->', $this->modifyValue($key, $value), $template);
                    }
                }
                return $template;
            }
    
            /**
             * JDovi42Templates::modifyValue()
             * 
             * A változó értékének módosítása - ez felülírandó a konkrét alkalmazás során, ha szükség van rá
             * 
             * @param $key A változó kulcsa 
             * @param $value A változó értéke 
             * @return $value Módosított érték
             */
            function modifyValue($key, $value) {
                /*
                if (!empty($key)) {
                    switch ($key) { 
                    	case 'timestamp':
                    	   $value = date_i18n(get_option( 'date_format' ).' '.get_option( 'time_format' ), $value);
                           break;
                    
                    	default :
                    }                
                }
                */
                return $value;
            }
    
    
            /**
             * JDovi42Templates::conditionalTemplate()
             * 
             * A sablonban deklarált feltételrendszernek megfelelően formázza a sablont. (Törli a felesleges értékeket.)
             * 
             * @param HTML $template Az aktuálisan használt sablon HTML tartalma
             * @return void
             */
            private function conditionalTemplate($template, $data = array()) {
                
                // Ha nem kap adattartalmat, akkor az osztály adatait tartalmazza
                if (empty($data)) {
                    $data = $this->options;
                }
                // Ha ez egy objektum, akkor tömböt csinálunk belőle.
                if (is_object($data)) { $data = (array) $data; }
    
                $data = wp_parse_args($data, $this->globals);
                
                // Feltételekhez használt kapcsolók tömbje
                $conditionals = array();
                $regex = '/<!-- if (\s*.*?) -->/s';
                preg_match_all( $regex, $template, $matches);
                if (!empty($matches)) {
                    foreach ($matches[1] as $name) {
                        if (!in_array($name, $conditionals)) {
                            $conditionals[] = $name;
                        }
                    }
                }
    
                // Minden feltételhez kapcsolt változóra vizsgáljuk a sablont
                foreach ($conditionals as $conditional) {
    
                    $regex = '/<!-- if '.$conditional.' -->(\s*.*?)<!-- endif '.$conditional.' -->/s';
                    preg_match_all( $regex, $template, $matches);
                    if (!empty($matches)) {
                        for ($i = 0; $i <= count($matches[0])-1; $i++) {
                            $content = explode('<!-- else '.$conditional.' -->', $matches[1][$i]);
                            if (isset($data[$conditional]) and !empty($data[$conditional])) {
                                $template = str_replace($matches[0][$i], $content[0], $template);
                            } else {
                                if (isset($content[1])) {
                                    $template = str_replace($matches[0][$i], $content[1], $template);
                                } else {
                                    $template = str_replace($matches[0][$i], '', $template);
                                }
                            }
                        }
                    }
                } 
                return $template;           
            }

            /**
             * JDovi42_Templates::addError()
             * Add text to errors array
             * 
             * @param mixed $text
             */
            function addError($text) {
                $this->errors[] = "\t" . $text;
            }

            /**
             * JDovi42_Templates::getErrors()
             * 
             * @return errors Pure errors list
             */
            function getErrors() {
                
                $html = '';
                if (!empty($this->errors)) {
                    $html = implode("\r\n", $this->errors); 
                    $html = str_replace("\t", '', $html);
                }
                return $html;
            }

            /**
             * JDovi42_Templates::getErrorsHTML()
             * 
             * @return HTML formatted errors list
             */
            function getErrorsHTML() {
                
                $html = '';
                if (!empty($this->errors)) {
                    $html = "\r\n" . '<!-- ['.$this->ID.']: ' . "\r\n" .
                            implode("\r\n", $this->errors) . 
                            ' -->' . "\r\n";
                }
                return $html;
            }

    
            /**
             * JDovi42Templates::prepareContent()
             * 
             * A generált HTML tartalom visszaadása
             * 
             * @return HTML
             */
            function prepareContent() {
                
                // Betölti a sablont a megfelelő helyről 
                $template = $this->getTemplate($this->template_file);
                
                // CSS és JS jájlok betöltése
                $this->loadCSSandJS();
    
                // HTML tartalom renderelése
                $this->content = $this->renderContent($template);
    
                $html = "\r\n";
                if (WP_DEBUG) {
                    $html .= $this->getErrorsHTML();
                } 
                $html .= $this->content;
                
                return $html;
            }
    
        }
    }